package com.inspect.metadata.patrolpointmnt.service;

import com.inspect.metadata.patrolpointmnt.domain.BasedataStdPatrolPoint;
import com.inspect.metadata.patrolpointmnt.domain.model.InsertStdModel;

import java.util.List;

public interface IBasedataStdPatrolPointService {
    BasedataStdPatrolPoint selectBasedataStdPatrolPointByStdPatrolPointId(Long patrolPointId);

    List<BasedataStdPatrolPoint> selectBasedataStdpPatrolPointList(BasedataStdPatrolPoint patrolPoint);

    int insertBasedataStdPatrolPoint(BasedataStdPatrolPoint patrolPoint);

    int updateBasedataStdPatrolPoint(BasedataStdPatrolPoint patrolPoint);

    int deleteBasedataStdPatrolPointByStdPatrolPointIds(Long[] patrolPointIds);

    int deleteBasedataStdPatrolPointByStdPatrolPointId(Long patrolPointId);

    int addBasedataStdPatrolPointForList(List<InsertStdModel> stdModels);
}
